/*
 *    SimpleGraphicalClient.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.nzdl.gsdl.SimpleGraphicalClient;

import java.util.Set;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.Vector;

// local libraries

import org.nzdl.gsdl.service.NzdlService;
import org.nzdl.gsdl.service.NzdlCollectionInfo;
/**
 * Class 
 *
 * A class to 
 * 
 * @author Dave Nichols (daven@cs.waikato.ac.nz)
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2184 $
 * @see org.nzdl.gsdl.service.SimpleGraphicalClient.SimpleGraphicalClient
 */

class CSModel extends Object
{

  Vector collectionList;
  String currentCollection;
  ResultModel results;
  NzdlService nzdl;
    

  public CSModel(NzdlService newNzdl)
  {
    nzdl = newNzdl;
    results = new ResultModel();
    collectionList = new Vector();
    currentCollection = "";
    initialiseCollection();
  }
  
  
  public NzdlService getNzdlService()
  {
    return nzdl;
  }

	
	

  /* Result methods */

  public ResultModel getResultsModel()
  {
    return results;
  }
	
  public void addResult(Result result)
  {
    results.add(result);
  }
	
  public void clearResults()
  {
    //System.out.println("Clearing resultModel");
    results.clear();
  }

  /* collectionList methods */

  public Vector getCollectionList()
  {
    return collectionList;
  }

  public int getCollectionListSize() 
  {
    return collectionList.size();
  }
	
  public void addCollection(String collection)
  {
    collectionList.add(collection);
  }

  public void setCurrentCollection(String collection )
  {
    currentCollection = collection;
  }

  public String getCurrentCollection()
  {
    return currentCollection;
  }

  public String getFirstCollection()
  {
    if (collectionList.size() > 0)
      return (String)collectionList.get(0);
    else
      return "no collections";
  }
  /**
   *
   */
  public void initialiseCollection() 
  {
    Set collectionSet = nzdl.getCollectionSet(); // set of strings of collection titles
    TreeSet orderedCollectionSet = new TreeSet(collectionSet);
    // TreeSet automatically puts strings in alphabetical order
    System.out.println("Number of collections on the server: " + 
		       orderedCollectionSet.size());
    Iterator collectionSetIterator = orderedCollectionSet.iterator();
    while (collectionSetIterator.hasNext() ) {
      // object -> String
      String collectionName = (String)collectionSetIterator.next(); 
      NzdlCollectionInfo info = nzdl.getCollectionInfo(collectionName);
      //System.err.println(info);
      if (info != null && 
	  info.getBuildDate() != 0) {
	addCollection(collectionName);
	System.err.println("Adding " + collectionName + " to collection list...");
      } // end if
      else {
	System.err.println("Collection " + collectionName + " getBuildDate() returned 0");
      } // end else
    } // end while	

    

    setCurrentCollection(getFirstCollection()); //assume at least one collection
    
  }
	
	
}
