/*
 *    TODO.java
 *    Copyright (C) 2000 Stuart Yeates 
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// the package we're in
package org.nzdl.gsdl.ptp;

// java standard library classes used
import java.io.IOException;
import java.io.Reader;
import java.io.InputStreamReader;
import java.io.Writer;
import java.io.OutputStreamWriter;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.Serializable;
//import java.io.;
import java.util.*;
import java.net.*;

/**
 * Class Connection prepresents one end of a connection between two peers in a Peer-To-Peer Network.
 *
 * Handles all protocol work except parsing of Locators.
 *
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2124 $
 * @see org.nzdl.gsdl.ptp.Node
 * @see org.nzdl.gsdl.ptp.Poller
 * @see org.nzdl.gsdl.ptp.Listener
 * 
 */

public class Connection extends Thread
{
  // queries ...
  public static final String WHOAREYOU = "WHO ARE YOU?";
  public static final String WHODOYOUKNOW = "WHO DO YOU KNOW?";
  public static final String HELPREQUEST = "HELP?";
  public static final String WHATVERSIONAREYOU = "WHAT VERSION ARE YOU?";
  public static final String OUT = "OUT.";

  // responses
  public static final String IAMIOR = "I AM IOR:";
  public static final String IAMHTTP = "I AM HTTP:";
  public static final String IAMP2P = "I AM P2P:";

  public static final String IKNOWIOR = "I AM IOR:";
  public static final String IKNOWHTTP = "I AM HTTP:";
  public static final String IKNOWP2P = "I AM P2P:";

  public static final String HELPRESPONSE = "HELP "; // note the space ...

  public static final String VERSION = "VERSION ";

  /** Are we a server ? (false indicates we're a client) */
  boolean server = false;
 
  /** Has the connection been closed ? */
  boolean connectionClosed = false;
  /** the port we're connected to */
  int port = 0;
  /** the host we're connected to */
  String remoteHostName = "";
  /** the host we're connected to */
  InetAddress  remoteHost = null;

  /** The underlying socket */
  Socket socket = null;
  /** The input from the socket */
  Reader input = null;
  /** The output to the socket */
  Writer output = null;
 
  Connection(String host, int port) {
    new Thread("Client") ;
    server = false;

    this.port = port;
    this.remoteHostName = host;
    try {
      remoteHost = InetAddress.getByName(remoteHostName);
      socket = new Socket(remoteHost, port, InetAddress.getLocalHost(), 4567);
    } catch (Exception exception) {
      System.err.println("caught exception: "+exception);
    }
    try {
      input = new InputStreamReader(new BufferedInputStream(socket.getInputStream()));
      output = new OutputStreamWriter(new BufferedOutputStream(socket.getOutputStream()));  
    } catch (IOException exception) {
      System.err.println("caught exception: "+exception);
    }
  }
  Connection(Socket socket) {
    new Thread("Server") ;
    server = true;
    setPriority(MIN_PRIORITY);

    this.socket = socket;
    port = socket.getPort();
    remoteHost = socket.getInetAddress();
    remoteHostName = remoteHost.getHostName();

    try {
      input = new InputStreamReader(new BufferedInputStream(socket.getInputStream()));
      output = new OutputStreamWriter(new BufferedOutputStream(socket.getOutputStream()));  
    } catch (IOException exception) {
      System.err.println("caught exception: "+exception);
    }
  }

  String readline() throws IOException {
    StringBuffer buff = new StringBuffer(1000);
    while (true) {
      int c = input.read();
      if (c == '\n' || c == '\r')
	return buff.toString();
      buff.append((char)c);
    }
  }

  public void run() {
    try {
      while (!server || !connectionClosed) {
	String line = readline();    
	reply(line);
      }
    } catch (Exception exception) {
      System.err.println("caught exception:" + exception);
      connectionClosed = true;
    }
  }
  
  int reply(String line) throws IOException {

    String first = line.substring(0,WHOAREYOU.length());
    if (first.compareToIgnoreCase(WHOAREYOU) == 0) {
      System.err.println("We were asked who we are ...");
      return whoWeAre();
    }

    first = line.substring(0,WHODOYOUKNOW.length());
    if (first.compareToIgnoreCase(WHODOYOUKNOW) == 0) {
      System.err.println("We were asked who we know ...");
      return whoWeKnow();
    }

    first = line.substring(0,HELPREQUEST.length());
    if (first.compareToIgnoreCase(HELPREQUEST) == 0) {
      System.err.println("We were asked for help ...");
      return help();
    }

    first = line.substring(0,WHATVERSIONAREYOU.length());
    if (first.compareToIgnoreCase(WHATVERSIONAREYOU) == 0) {
      System.err.println("We were asked what verrsion we are ...");
      return whatVersionAreWe();
    }

    first = line.substring(0,OUT.length());
    if (first.compareToIgnoreCase(OUT) == 0) {
      System.err.println("We were asked to close the connection ...");
      connectionClosed = true;
      return gracefulClose();
    }
    System.err.println("Nothing matched \"" + line + "\" sending help ...");
    return help();
  }

  int gracefulClose() 
  {
    return -1;
  }


  int whoWeAre() throws IOException {
    output.write("I AM IOR:1234567890");
    output.write("I AM HTTP://www.someplace");
    output.write("I AM P2P:www.someplace:4444");
    return 3;
  }

  int whoWeKnow() throws IOException {
    output.write("I KNOW IOR:12345678901");
    output.write("I KNOW IOR:12345678902");
    output.write("I KNOW HTTP://www.someplace.else");
    output.write("I KNOW HTTP://www.someplace.elsewhere");
    output.write("I KNOW P2P:www.someplace:4444:A");
    output.write("I KNOW P2P:www.someplace:4444:B");
    return 6;
  }

  int help() throws IOException {
    output.write("HELP: This is NZDL P2P protocol $Revision: 2124 $");
    output.write("HELP: There are five valid commands:");
    output.write("HELP: WHO ARE YOU?");
    output.write("HELP: WHO DO YOU KNOW?");
    output.write("HELP: WHAT VERSION ARE YOU?");
    output.write("HELP: HELP");
    output.write("HELP: OUT");
    return 7;
  }

  int whatVersionAreWe() throws IOException {
    output.write("VERSION: Protocol: NZDL-P2P-0.1 ");
    output.write("VERSION: Implementation: $Revision: 2124 $");
    return 2;
  }

  /**
   * Tests...
   *
   * 
   * @exception java.io.IOException when ...
   * @param args the arguments ...
   */

  public static void main(String args[]) throws IOException
  {

  }
}

